//! IR test helpers.

use crate::executor::bucket::Buckets;
use crate::executor::engine::cartridge::backend::sql::ir::PatternWithParams;
use crate::executor::engine::mock::RouterConfigurationMock;
use crate::executor::ir::ExecutionPlan;
use crate::frontend::sql::ast::AbstractSyntaxTree;
use crate::frontend::Ast;
use crate::ir::value::Value;
use crate::ir::Plan;

/// Compiles an SQL query to IR plan.
#[allow(dead_code)]
pub fn sql_to_ir(query: &str, params: &[Value]) -> Plan {
    let metadata = &RouterConfigurationMock::new();
    let ast = AbstractSyntaxTree::new(query).unwrap();
    let mut plan = ast.resolve_metadata(metadata).unwrap();
    plan.bind_params(params).unwrap();
    plan
}

/// Compiles and transforms an SQL query to a new parameterized SQL.
#[allow(dead_code)]
pub fn sql_to_sql(
    query: &str,
    params: &[Value],
    f_transform: &dyn Fn(&mut Plan),
) -> PatternWithParams {
    let mut plan = sql_to_ir(query, params);
    f_transform(&mut plan);
    let ex_plan = ExecutionPlan::from(plan);
    let top_id = ex_plan.get_ir_plan().get_top().unwrap();
    let nodes = ex_plan.get_sql_order(top_id).unwrap();
    ex_plan.syntax_nodes_as_sql(&nodes, &Buckets::All).unwrap()
}
